Package: AdvancedR
Type: Package
Title: Advanced R Programming for Bioinformatics
Version: 0.0.3
Author: Martin Morgan <mtmorgan@fhcrc.org>,
        Patrick Aboyoun <paboyoun@fhcrc.org>,
        Marc Carlson <mcarlson@fhcrc.org>,
        Seth Falcon <sfalcon@fhcrc.org>,
        Nishant Gopalakrishnan <ngopalak@fhcrc.org>,
        Chao-Jen Wong <cwon2@fhcrc.org>
Maintainer: Martin Morgan <mtmorgan@fhcrc.org>
License: Artistic-2.0
LazyLoad: yes
Description: This package contains material for 'Advanced R
    Programming for Bioinformatics', offered in Seattle, May, 2010.
Suggests: codetoolsBioC, RUnit, Biobase, limma, IRanges, Biostrings,
          GenomicRanges, GenomicFeatures, foreach, doMC, RSQLite, ncdf
Packaged: 2010-05-24 20:52:05 UTC; mtmorgan
