Package: nidemo
Type: Package
Title: Native Interfaces Demo Package
Version: 0.0.1
Author: Seth Falcon <sfalcon@fhcrc.org>
Maintainer: Seth Falcon <sfalcon@fhcrc.org>
License: Artistic-2.0
LazyLoad: yes
Description: A package to demonstrate R's native interfaces for
    use in the course Advanced Programming for Bioinformatics',
    offered in Seattle, May, 2010.
