useDynLib("nidemo",
          .nid_avg = nid_avg,
          .nid_alpha_freq = nid_alpha_freq,
          .nid_which = nid_which,
          .nid_buggy_freq = nid_buggy_freq,
          .nid_make_freq_data = nid_make_freq_data,
          .nid_update_freq = nid_update_freq,
          .nid_report_freq = nid_report_freq)

exportPattern("^[^\\.]")
