system("hostname")
fname0 <-
    system.file("AdvancedR", "extdata", "gwas_2.csv", package="AdvancedR")
snp0 <- function(i, gwas) {
    snps <- gwas[,-(1:3)]
    glm(CaseControl ~ Age + Sex + snps[,i], family=binomial, 
        data=gwas)$coef
}

gwas <- read.csv(fname0, row.names=1)
result <- lapply(1:20, snp0, gwas)
result[[1]]
