scriptPath <- function(scriptName)
    if (missing(scriptName)) {
        system.file("script", package="AdvancedR")
    } else {
        system.file("script", scriptName, package="AdvancedR")
    }

readScript <- function(scriptName)
    noquote(readLines(scriptPath(scriptName)))
