%\VignetteDepends{}
%\VignetteIndexEntry{}
%\VignettePackage{AdvancedR}

\documentclass{beamer}

\usepackage{Bioconductor}
\SweaveOpts{keep.source=TRUE}

\title{Presentation title}
\author{Author}
\institute[FHCRC]{Fred Hutchinson Cancer Research Center}
\date{20-21 May, 2010}

\begin{document}

<<setup, echo=FALSE>>=
options(width = 40)
@ 

\frame{\titlepage}

\section[Outline]{}
\frame{\tableofcontents}

\section{Introduction}
\begin{frame}{frame title}
\end{frame}

\begin{frame}[fragile]{Subsetting and non-specific filtering}
<<code-block>>=
## use [fragile] in frames with Sweave
x <- 1:10
@ 
\end{frame}

\section{Resources}

\begin{frame}{Resources}
  %% Include key resources -- urls, books, ...
\end{frame}

\end{document}
