\name{AdvancedR-package}
\alias{AdvancedR-package}
\docType{package}
\title{

  Advanced R Programming for Bioinformatics, Seattle, May, 2010

}
\description{

  This package contains course material for \sQuote{Advanced R
    Programming for Bioinformatics}, offered in Seattle, 20-21 May,
  2010.

}
\details{

  Use \code{browseVignettes(package='AdvancedR')} for slides and other
  resources, including the \R{} code used in each slide.

  See \code{?scriptPath} to access scripts used to illustrate the
  slides or exercises.

  The \code{nidemo} package used for native interfaces is located at
  \code{system.file("packages", package="AdvancedR")}

}

\examples{
packageDescription('AdvancedR')
system.file("packages", package="AdvancedR") ## nidemo location
}
\keyword{package}
